#!/bin/sh
#
# UCSD p-System cross compiler
# Copyright (C) 2010-2012 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# you option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>
#

TEST_SUBJECT="ucsdpsys_osmakgen vs system.syntax"
. test_prelude

TAB=`awk 'BEGIN{printf("%c",9)}' /dev/null`

sed "s|{TAB}|$TAB|g" > ok << 'fubar'
#
# This file is generated by the ucsdpsys_osmakgen(1) command, from the
# ucsd-psystem-xc project.  The ucsd-psystem-xc project is distributed under the
# terms and conditions of the GNU GPL version 3.  As a special exception, the
# contents of this file are not subject to those terms and conditions.
#

# values provided by the ./configure script
prefix = @prefix@
exec_prefix = @exec_prefix@
datadir = $(DESTDIR)@datadir@
datarootdir = $(DESTDIR)@datarootdir@
sysconfdir = $(DESTDIR)@sysconfdir@

all: stage1.disks

host = klebsch

arch := $(shell ucsdpsys_osmakgen --arch-from-host $(host))

# cross compiler flags
XCFLAGS = --arch=$(arch) -Wno-shadow

filenames = compiler/error-messages.text compiler/main.text

stage1/$(host)/codefiles/compiler/main.code: compiler/main.text
{TAB}@mkdir -p stage1/$(host)/codefiles/compiler
{TAB}ucsdpsys_compile $(XCFLAGS) -fignore-undefined-segment-zero -Icompiler \
{TAB}{TAB}-o stage1/$(host)/codefiles/compiler/main.code \
{TAB}{TAB}compiler/main.text
{TAB}ucsdpsys_librarian --file=stage1/$(host)/codefiles/compiler/main.code \
{TAB}{TAB}--remove-system-segments

stage1.disks: stage1/$(host)/system.vol

stage2.disks: stage2/$(host)/system.vol

stage3.disks: stage3/$(host)/system.vol

stage1/$(host)/system/system.compiler: \
{TAB}{TAB}stage1/$(host)/codefiles/compiler/main.code
{TAB}@mkdir -p stage1/$(host)/system
{TAB}cp stage1/$(host)/codefiles/compiler/main.code \
{TAB}{TAB}stage1/$(host)/system/system.compiler

stage1/$(host)/system/system.syntax: compiler/error-messages.text
{TAB}@mkdir -p stage1/$(host)/system
{TAB}cp compiler/error-messages.text stage1/$(host)/system/system.syntax

# build stage one system disk image

stage1/$(host)/system.vol: stage1/$(host)/system/system.compiler \
{TAB}{TAB}stage1/$(host)/system/system.syntax
{TAB}@mkdir -p stage1/$(host)
{TAB}ucsdpsys_mkfs --arch=$(arch) -B256 --label=system \
{TAB}{TAB}stage1/$(host)/system.vol
{TAB}ucsdpsys_disk --file stage1/$(host)/system.vol --text --put \
{TAB}{TAB}stage1/$(host)/system/system.compiler \
{TAB}{TAB}stage1/$(host)/system/system.syntax
{TAB}ucsdpsys_disk --file stage1/$(host)/system.vol --crunch --list \
{TAB}{TAB}--sort=name

stage2/$(host)/system/system.compiler: \
{TAB}{TAB}stage2/$(host)/codefiles/compiler/main.code
{TAB}@mkdir -p stage2/$(host)/system
{TAB}cp stage2/$(host)/codefiles/compiler/main.code \
{TAB}{TAB}stage2/$(host)/system/system.compiler

stage2/$(host)/system/system.syntax: compiler/error-messages.text
{TAB}@mkdir -p stage2/$(host)/system
{TAB}cp compiler/error-messages.text stage2/$(host)/system/system.syntax

# build stage two system disk image

stage2/$(host)/system.vol: stage2/$(host)/system/system.compiler \
{TAB}{TAB}stage2/$(host)/system/system.syntax
{TAB}@mkdir -p stage2/$(host)
{TAB}ucsdpsys_mkfs --arch=$(arch) -B256 --label=system \
{TAB}{TAB}stage2/$(host)/system.vol
{TAB}ucsdpsys_disk --file stage2/$(host)/system.vol --text --put \
{TAB}{TAB}stage2/$(host)/system/system.compiler \
{TAB}{TAB}stage2/$(host)/system/system.syntax
{TAB}ucsdpsys_disk --file stage2/$(host)/system.vol --crunch --list \
{TAB}{TAB}--sort=name

stage3/$(host)/system/system.compiler: \
{TAB}{TAB}stage3/$(host)/codefiles/compiler/main.code
{TAB}@mkdir -p stage3/$(host)/system
{TAB}cp stage3/$(host)/codefiles/compiler/main.code \
{TAB}{TAB}stage3/$(host)/system/system.compiler

stage3/$(host)/system/system.syntax: compiler/error-messages.text
{TAB}@mkdir -p stage3/$(host)/system
{TAB}cp compiler/error-messages.text stage3/$(host)/system/system.syntax

# build stage three system disk image

stage3/$(host)/system.vol: stage3/$(host)/system/system.compiler \
{TAB}{TAB}stage3/$(host)/system/system.syntax
{TAB}@mkdir -p stage3/$(host)
{TAB}ucsdpsys_mkfs --arch=$(arch) -B256 --label=system \
{TAB}{TAB}stage3/$(host)/system.vol
{TAB}ucsdpsys_disk --file stage3/$(host)/system.vol --text --put \
{TAB}{TAB}stage3/$(host)/system/system.compiler \
{TAB}{TAB}stage3/$(host)/system/system.syntax
{TAB}ucsdpsys_disk --file stage3/$(host)/system.vol --crunch --list \
{TAB}{TAB}--sort=name

clean: clean-files
clean-files:
{TAB}rm -f stage1/$(host)/codefiles/compiler/main.code
{TAB}rm -f stage1/$(host)/system.vol stage1/$(host)/system/system.compiler
{TAB}rm -f stage1/$(host)/system/system.syntax stage2/$(host)/system.vol
{TAB}rm -f stage2/$(host)/system/system.compiler
{TAB}rm -f stage2/$(host)/system/system.syntax stage3/$(host)/system.vol
{TAB}rm -f stage3/$(host)/system/system.compiler
{TAB}rm -f stage3/$(host)/system/system.syntax

distclean: clean dist-clean-files dist-clean-dirs
dist-clean-files:
{TAB}rm -f Makefile config.log config.status
dist-clean-dirs:
{TAB}rm -rf stage1

rebuild-makefile:
{TAB}ucsdpsys_osmakgen $(filenames)

install: stage1.install

stage1.install: stage1.disks
{TAB}mkdir -p $(datadir)/ucsd-psystem-os
{TAB}cp -r stage1/$(host)/diagnostic $(datadir)/ucsd-psystem-os/diagnostic
{TAB}cp stage1/$(host)/diagnostic.vol \
{TAB}{TAB}$(datadir)/ucsd-psystem-os/diagnostic.vol
{TAB}cp -r stage1/$(host)/system $(datadir)/ucsd-psystem-os/system
{TAB}cp stage1/$(host)/system.vol $(datadir)/ucsd-psystem-os/system.vol
{TAB}cp -r stage1/$(host)/utility $(datadir)/ucsd-psystem-os/utility
{TAB}cp stage1/$(host)/utility.vol $(datadir)/ucsd-psystem-os/utility.vol

stage2.install: stage2.disks
{TAB}mkdir -p $(datadir)/ucsd-psystem-os
{TAB}cp -r stage2/$(host)/diagnostic $(datadir)/ucsd-psystem-os/diagnostic
{TAB}cp stage2/$(host)/diagnostic.vol \
{TAB}{TAB}$(datadir)/ucsd-psystem-os/diagnostic.vol
{TAB}cp -r stage2/$(host)/system $(datadir)/ucsd-psystem-os/system
{TAB}cp stage2/$(host)/system.vol $(datadir)/ucsd-psystem-os/system.vol
{TAB}cp -r stage2/$(host)/utility $(datadir)/ucsd-psystem-os/utility
{TAB}cp stage2/$(host)/utility.vol $(datadir)/ucsd-psystem-os/utility.vol

stage3.install: stage3.disks
{TAB}mkdir -p $(datadir)/ucsd-psystem-os
{TAB}cp -r stage3/$(host)/diagnostic $(datadir)/ucsd-psystem-os/diagnostic
{TAB}cp stage3/$(host)/diagnostic.vol \
{TAB}{TAB}$(datadir)/ucsd-psystem-os/diagnostic.vol
{TAB}cp -r stage3/$(host)/system $(datadir)/ucsd-psystem-os/system
{TAB}cp stage3/$(host)/system.vol $(datadir)/ucsd-psystem-os/system.vol
{TAB}cp -r stage3/$(host)/utility $(datadir)/ucsd-psystem-os/utility
{TAB}cp stage3/$(host)/utility.vol $(datadir)/ucsd-psystem-os/utility.vol
fubar
test $? -eq 0 || no_result

mkdir -p compiler
test $? -eq 0 || no_result

touch compiler/main.text compiler/error-messages.text
test $? -eq 0 || no_result

ucsdpsys_osmakgen -o test.makefile.out -b compiler/main.text \
        compiler/error-messages.text
test $? -eq 0 || fail

diff ok test.makefile.out
test $? -eq 0 || fail

#
# The functionality exercised by this test worked.
# No other assertions are made.
#
pass
# vim: set ts=8 sw=4 et :
